--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--         CORPS DU PAQUETAGE DES ARBRES BALANCES           --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Arbres_Balances is
  procedure Initialiser_Feuille(Le_Noeud: in out Type_Noeud;
                                 Val : in Element) is
  begin
    Tronquer (Le_Noeud.Contenu, 0); -- vider contenu eventuel
    Tronquer (Le_Noeud.Sous_Arbres, 0);
    Prolonger (Le_Noeud.Contenu, Val);    -- y mettre la valeur
  end Initialiser_Feuille;

  procedure Initialiser_Noeud (Le_Noeud : in out Type_Noeud;
                         Val_Separ : in Element;
                         Gauche, Droite : in Id_Noeud) is
  begin
    Tronquer (Le_Noeud.Contenu, 0); -- vider contenu eventuel
    Tronquer (Le_Noeud.Sous_Arbres, 0);
    Prolonger (Le_Noeud.Contenu, Val_Separ);  -- mettre valeur
    Prolonger (Le_Noeud.Sous_Arbres, Gauche); -- mettre les 
    Prolonger (Le_Noeud.Sous_Arbres, Droite); -- deux fils
  end Initialiser_Noeud;

  function Longueurs_Compatibles (Le_Noeud : in Type_Noeud) 
                                            return Boolean is
    LC : Natural := Longueur (Le_Noeud.Contenu);
    LF : Natural := Longueur (Le_Noeud.Sous_Arbres);
  begin 
    return (LF = 0 or LF = LC + 1) and LC <= Degre_Max;
  end Longueurs_Compatibles;

  function Degre(Le_Noeud: in Type_Noeud) return Natural is
  begin
    return Longueur (Le_Noeud.Contenu);
  end Degre;

  function Est_Feuille (Le_Noeud : in Type_Noeud) 
                                           return Boolean is
  begin
    return Longueur (Le_Noeud.Sous_Arbres) = 0;
  end Est_Feuille;

  function Rang (Le_Noeud : in Type_Noeud;
                  La_Cle : in Cle) return Positive is
  begin
    return Recherche (Le_Noeud.Contenu, La_Cle);
  end Rang;

  function Ieme_Valeur (Le_Noeud : in Type_Noeud;
                     Indice : in Positive) return Element is
  begin
    return Ieme (Le_Noeud.Contenu, Indice);
  end Ieme_Valeur;

  function Ieme_Fils (Le_Noeud : in Type_Noeud;
                    Indice : in Positive) return Id_Noeud is
  begin
    return Ieme (Le_Noeud.Sous_Arbres, Indice);
  end Ieme_Fils;

  procedure Changer_Valeur (Le_Noeud : in out Type_Noeud;
                   Indice : in Positive; Val : in Element) is
  begin
    Changer_Ieme (Le_Noeud.Contenu, Indice, Val);
  end Changer_Valeur;

  procedure Inserer (Le_Noeud : in out Type_Noeud;
                 Indice : in Positive; Val : in Element) is
  begin
    if not Est_Feuille (Le_Noeud) then 
      raise Erreur_Specification; 
    end if;
    Inserer (Le_Noeud.Contenu, Indice, Val);
  end Inserer;

  procedure Inserer (Le_Noeud : in out Type_Noeud;
                      Indice : in Positive; Val : in Element;
                      Q : in Id_Noeud) is
  begin
    if Est_Feuille (Le_Noeud) then 
      raise Erreur_Specification; 
    end if;
    Inserer (Le_Noeud.Contenu, Indice, Val);
    Inserer (Le_Noeud.Sous_Arbres, Indice + 1, Q);
  end Inserer;

  procedure Supprimer (Le_Noeud : in out Type_Noeud;
                        Indice : in Positive) is
  begin
    Supprimer (Le_Noeud.Contenu, Indice);
    if not Est_Feuille (Le_Noeud) then
      Supprimer (Le_Noeud.Sous_Arbres, Indice + 1);
    end if;
  end Supprimer;

  procedure Eclater (Le_Noeud : in out Type_Noeud; 
                      En : in Positive;
                      Val_Separ : out Element;
                      Nouveau : in out Type_Noeud) is
    Degre_N : Positive := Degre (Le_Noeud);
  begin
    Val_Separ := Ieme (Le_Noeud.Contenu, En + 1);
    Tronquer (Nouveau.Contenu, 0);
    Tronquer (Nouveau.Sous_Arbres, 0);
    for K in En + 2..Degre_N loop
      Prolonger (Nouveau.Contenu, Ieme (Le_Noeud.Contenu, K));
    end loop;
    Tronquer (Le_Noeud.Contenu, En);
    if not Est_Feuille (Le_Noeud) then
      for K in En + 2..Degre_N + 1 loop
        Prolonger (Nouveau.Sous_Arbres, 
                   Ieme (Le_Noeud.Sous_Arbres, K));
      end loop;
      Tronquer (Le_Noeud.Sous_Arbres, En + 1);
    end if;
  end Eclater;

  procedure Regrouper (Le_Noeud_Pere : in out Type_Noeud;
                        Indice : in Positive;
                        Gauche, Droite: in out Type_Noeud) is
    Degre_G : Positive := Degre (Gauche);
    Degre_D : Positive := Degre (Droite);
  begin
    if Est_Feuille (Gauche) /= Est_Feuille (Droite) then
      raise Erreur_Specification;
    end if;
    Prolonger(Gauche.Contenu,
              Ieme (Le_Noeud_Pere.Contenu,Indice));
    for K in 1..Degre_D loop
      Prolonger (Gauche.Contenu, Ieme (Droite.Contenu, K));
    end loop;
    if not Est_Feuille (Gauche) then
      for K in 1..Degre_D + 1 loop
        Prolonger (Gauche.Sous_Arbres, 
                   Ieme (Droite.Sous_Arbres, K));
      end loop;
    end if;
    Supprimer (Le_Noeud_Pere.Contenu, Indice);
    Supprimer (Le_Noeud_Pere.Sous_Arbres, Indice + 1);
  end Regrouper;
 
  procedure Transfert_Droite_Gauche
                      (Le_Noeud_Pere : in out Type_Noeud;
                       Indice : in Positive; 
                       Gauche, Droite : in out Type_Noeud) is
  begin
    if Est_Feuille (Gauche) /= Est_Feuille (Droite) then
      raise Erreur_Specification;
    end if;
    Prolonger(Gauche.Contenu, Ieme (Le_Noeud_Pere.Contenu,Indice));
    Changer_Ieme(Le_Noeud_Pere.Contenu,Indice,Ieme(Droite.Contenu,1));
    Supprimer (Droite.Contenu, 1);
    if not Est_Feuille (Gauche) then
      Prolonger (Gauche.Sous_Arbres, Ieme (Droite.Sous_Arbres, 1));
      Supprimer (Droite.Sous_Arbres, 1);
    end if;
  end Transfert_Droite_Gauche;

  procedure Transfert_Gauche_Droite
                      (Le_Noeud_Pere : in out Type_Noeud;
                       Indice : in Positive; 
                       Gauche, Droite : in out Type_Noeud) is
    Degre_G : Positive := Degre (Gauche);
  begin
    if Est_Feuille (Gauche) /= Est_Feuille (Droite) then
      raise Erreur_Specification;
    end if;
    Inserer (Droite.Contenu, 1, Ieme (Le_Noeud_Pere.Contenu, Indice));
    Changer_Ieme (Le_Noeud_Pere.Contenu, Indice,
                  Ieme (Gauche.Contenu, Degre_G));
    Tronquer (Gauche.Contenu, Degre_G - 1);
    if not Est_Feuille (Gauche) then
      Inserer (Droite.Sous_Arbres, 1, 
               Ieme (Gauche.Sous_Arbres, Degre_G + 1));
      Tronquer (Gauche.Sous_Arbres, Degre_G);
    end if;
  end Transfert_Gauche_Droite;
end Arbres_Balances;
