--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--      CORPS DE LA PROCEDURE DADJONCTION A LA RACINE      --
--        DANS UN ARBRE BINAIRE DE RECHERCHE POINTE         --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Ada.Tags;
with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits; 
procedure Arbres_Binaires.De_Recherche.Ajouter_Racine
                         (Dans:in out Arbre_Rech; Val:in Element) is
  use Ada.Tags;
  Cval : Cle := La_Cle (Val);
  Nouveau : Pt_Noeud; -- o raccrocher les sous-arbres
  Gauche : Position; -- gauche
  Droite : Position; -- droit 
  Courant : Pt_Noeud := Dans.Chemin(1).Le_Noeud;
begin
  if Arbre_Rech'Class(Dans)'Tag /= Arbre_Rech'Tag then
    raise Erreur_Specification;
  end if;
  Nouveau:= new Noeud_Rech'(Contenu => Val, others => null);
  Gauche := (Nouveau, True);
  Droite := (Nouveau, False);
  Dans.Sommet := 1;
  Dans.Chemin(1).Le_Noeud := Nouveau;
  while Courant /= null loop
    if Cval = La_Cle (Noeud_Rech(Courant.all).Contenu) then
        -- existe deja dans larbre. On finit la structure, 
        -- en gardant lancienne valeur dans le nouveau noeud
      Noeud_Rech(Nouveau.all).Contenu := 
                                   Noeud_Rech(Courant.all).Contenu;
      Raccrocher (Gauche, Courant.Gauche);
      Raccrocher (Droite, Courant.Droite);
      Liberer (Courant);            -- liberer lancien noeud
      raise Erreur_Specification;
    elsif Cval < La_Cle (Noeud_Rech(Courant.all).Contenu) then
      Raccrocher (Droite, Courant); -- droite := courant
      Descendre (Droite, Courant, True); --coupure(C.G, G, C.G)
    else
      Raccrocher (Gauche, Courant); -- gauche := courant
      Descendre (Gauche, Courant, False); --coupure(C.D, C.D, D)
    end if;
  end loop;
  Raccrocher (Gauche, null);
  Raccrocher (Droite, null);
end Arbres_Binaires.De_Recherche.Ajouter_Racine;
