--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--        CORPS DU PAQUETAGE DES ARBRES AVL POINTES         --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Ada.Tags;
with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits; 
package body Arbres_Binaires.De_Recherche.AVL is
  procedure Reequilibrage (Le_Noeud : in out Pt_Noeud) is
    D : Desequilibre;
    Result : Pt_Noeud;
    procedure Rotation_Droite(Le_Noeud : in out Pt_Noeud) is
    begin
      Result := Le_Noeud.Gauche;
      Le_Noeud.Gauche := Result.Droite;
      Result.Droite := Le_Noeud;
      Le_Noeud := Result;
    end Rotation_Droite;
    procedure Rotation_Gauche(Le_Noeud : in out Pt_Noeud) is
    begin
      Result := Le_Noeud.Droite;
      Le_Noeud.Droite := Result.Gauche;
      Result.Gauche := Le_Noeud;
      Le_Noeud := Result;
    end Rotation_Gauche;
  begin
    if Noeud_AVL(Le_Noeud.all).Deseq = 2 then
      if Noeud_AVL(Le_Noeud.Gauche.all).deseq = -1 then
                                    -- rotation gauche droite
        Rotation_Gauche (Le_Noeud.Gauche);
        Rotation_Droite (Le_Noeud);
        D := Noeud_AVL(Le_Noeud.all).Deseq;
        Noeud_AVL(Le_Noeud.Gauche.all).Deseq := - (D - abs D) / 2;
        Noeud_AVL(Le_Noeud.Droite.all).Deseq := - (D + abs D) / 2;
        Noeud_AVL(Le_Noeud.all).Deseq := 0;
      else                                 -- rotation  droite
        Rotation_Droite (Le_Noeud);
        Noeud_AVL(Le_Noeud.Droite.all).Deseq := 
                                1 - Noeud_AVL(Le_Noeud.all).Deseq;
        Noeud_AVL(Le_Noeud.all).Deseq := 
                                Noeud_AVL(Le_Noeud.all).Deseq - 1;
      end if;
    elsif Noeud_AVL(Le_Noeud.all).Deseq = -2 then
      if Noeud_AVL(Le_Noeud.Droite.all).Deseq = 1 then
                                    -- rotation droite gauche
        Rotation_Droite (Le_Noeud.Droite);
        Rotation_Gauche (Le_Noeud);
        D := Noeud_AVL(Le_Noeud.all).Deseq;
        Noeud_AVL(Le_Noeud.Gauche.all).Deseq := - (D - abs D) / 2;
        Noeud_AVL(Le_Noeud.Droite.all).Deseq := - (D + abs D) / 2;
        Noeud_AVL(Le_Noeud.all).Deseq := 0;
      else                                 -- rotation  gauche
        Rotation_Gauche (Le_Noeud);
        Noeud_AVL(Le_Noeud.Gauche.all).Deseq := 
                               -1 - Noeud_AVL(Le_Noeud.all).Deseq;
        Noeud_AVL(Le_Noeud.all).Deseq := 
                               Noeud_AVL(Le_Noeud.all).Deseq + 1;
      end if;
    end if; 
  end Reequilibrage;

  procedure Ajouter (Dans: in out Arbre_AVL; Val: in Element) is
    use Ada.Tags;
    C : Tab_Pt_Noeud renames Dans.Chemin;
    S : Natural renames Dans.Sommet;
    Le_Noeud : Pt_Noeud;
    Cval : Cle := La_Cle (Val);
    procedure Ajuster_Chemin (En : in Positive) is 
    begin                                 -- gestion du chemin
      if Noeud_AVL(C(En).Le_Noeud.all).Deseq = 2 then 
        if not C(En + 1).A_Gauche then  -- rot. gauche droite
          if S = En + 2 then S := S - 1; -- 2 noeuds en moins
          elsif not C(En + 2).A_Gauche then C(En + 1) := C(En);
          end if;           -- un noeud disparait du chemin
          C(En) := C(En + 2);
        else                        -- rotation droite
          C(En..En + 1) := C(En + 1..En + 2);
        end if;
      elsif Noeud_AVL(C(En).Le_Noeud.all).Deseq = -2 then 
        if C(En + 1).A_Gauche then  -- rot. droite gauche 
          if S = En + 2 then S := S - 1;
          elsif C(En + 2).A_Gauche then C(En + 1) := C(En);
          end if;
          C(En) := C(En + 2);
        else                        -- rotation gauche
         C(En..En + 1) := C(En + 1..En + 2);
        end if;
      else return; -- pas dajustement
      end if;
                     -- on remonte les autres noeuds du chemin
      C(En + 2..S - 1) := C(En + 3..S);
      S := S - 1;
    end Ajuster_Chemin;
  begin
    if Arbre_AVL'Class(Dans)'Tag /= Arbre_AVL'Tag then
      raise Erreur_Specification;
    end if; 
    Recherche_Locale (Dans, Sur => La_Cle (Val)); 
    if Trouve then raise Erreur_Specification; end if; 
    Raccrocher(Dans, new Noeud_AVL'(Contenu => Val, 
                                     Deseq => 0, 
                                     others => null)); 
    for I in reverse Natural range 1..S - 1 loop 
                         -- correction des dsquilibres 
      Le_Noeud := C(I).Le_Noeud;
      declare
        N : Noeud_AVL renames Noeud_AVL(Le_Noeud.all);
      begin
        if C(I).A_Gauche then N.Deseq := N.Deseq + 1;
        else N.Deseq := N.Deseq - 1;
        end if;
      end;
      Ajuster_Chemin (I);
      Reequilibrage (Le_Noeud);
      Raccrocher (Dans, Le_Noeud, En => I);
      exit when Noeud_AVL(Le_Noeud.all).Deseq = 0;-- mme hauteur
    end loop;
  end Ajouter;

  procedure Supprimer_Courant (Dans : in out Arbre_AVL) is
    use Ada.Tags;
    C : Tab_Pt_Noeud renames Dans.Chemin;
    S : Natural renames Dans.Sommet;
    Le_Noeud : Pt_Noeud;  -- noeud physiquement  supprimer
    Corriger_En : Positive;   -- endroits du chemin 
    procedure Ajuster_Chemin (En : in Positive) is 
      Le_Noeud : Pt_Noeud := C(En).Le_Noeud;
      A_Gauche : Boolean;
    begin                                 -- gestion du chemin
      if Noeud_AVL(Le_Noeud.all).Deseq = 2 then 
        Le_Noeud := Le_Noeud.Gauche; -- noeud insr
        if Noeud_AVL(Le_Noeud.all).Deseq = -1 then --rot. ga - dr
          Le_Noeud := Le_Noeud.Droite; -- cest celui-ci
        end if;
        A_Gauche := False; -- il est avant le supprim 
      elsif Noeud_AVL(Le_Noeud.all).Deseq = -2 then 
        Le_Noeud := Le_Noeud.Droite; -- noeud insr
        if Noeud_AVL(Le_Noeud.all).Deseq = 1 then -- rot. dr ga 
          Le_Noeud := Le_Noeud.Gauche; -- cest celui-ci
        end if;
        A_Gauche := True;  -- il est aprs le supprim
      else return;     -- pas de changement du chemin
      end if;
      if En <= S then      -- sur le chemin courant
        C(En + 1..S + 1) := C(En..S);     -- un noeud  insrer
        C(En) := (Le_Noeud, A_Gauche); 
        S := S + 1;
      end if;
    end Ajuster_chemin;
  begin 
    if Arbre_AVL'Class(Dans)'Tag /= Arbre_AVL'Tag then
      raise Erreur_Specification;
    end if;
    Supprimer (Dans, Corriger_En); 
    for I in reverse Natural range 1..Corriger_En - 1 loop 
                         -- correction des dsquilibres 
      Le_Noeud := C(I).Le_Noeud;
      declare
        N : Noeud_AVL renames Noeud_AVL(Le_Noeud.all);
      begin 
        if C(I).A_Gauche then N.Deseq := N.Deseq - 1;
        else N.Deseq := N.Deseq + 1;
        end if;
      end ;
      Ajuster_Chemin (I);
      Reequilibrage (Le_Noeud);
      Raccrocher (Dans, Le_Noeud, En => I);
      exit when Noeud_AVL(Le_Noeud.all).Deseq /= 0; -- mme hauteur
    end loop;
  end Supprimer_Courant;
end Arbres_Binaires.De_Recherche.AVL;
