--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--            PAQUETAGE DES ARBRES AVL POINTES              --
--                                                          --
-- Note: les arbres AVL sont des arbres binaires de         --
--    recherche qui sont H-equilibre, cest-e-dire tels que --
--        pour tout sous-arbre a, |h(g(a)) - h(d(a))| <= 1  --
--    le paquetage est un paquetage enfant de celui des     --
--        arbres binaires de recherche pointes, le type     --
--        Arbre_AVL tant derive du type Arbre_Rech. Il     --
--        herite donc des operations dexploration sur les  --
--        arbres binaires, ainsi que les operations des     --
--        arbres binaires de recherche. Il redefinit les    --
--        operations dadjonction et de suppression.        --
-- preconditions                                            --
--    Supprimer_Courant: not A_La_Fin(L_Arbre)              --
--    Ajouter (Dans, Val): La_Cle(Val) nappartient pas a   --
--        Dans                                              --
-- proprietes fonctionnelles                                --
--    Ajouter (Dans, Val) ajoute un nouveau noeud dans      --
--        larbre, avec comme contenu Val                   --
--    Supprimer_Courant (L_Arbre) supprime le noeud courant --
-- operations de service                                    --
--    Reequilibrage (Le_Noeud) corrige un desequilibre      --
--        local sur Le_Noeud                                --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

generic
package Arbres_Binaires.De_Recherche.AVL is 
  type Arbre_AVL is new Arbre_Rech with null record; 
  procedure Ajouter (Dans : in out Arbre_AVL; Val : in Element); 
  procedure Supprimer_Courant (Dans : in out Arbre_AVL);
private
  type Desequilibre is range -2 .. +2;
  type Noeud_AVL is new Noeud_Rech with 
    record
      Deseq : Desequilibre := 0;
    end record;
  procedure Reequilibrage (Le_Noeud : in out Pt_Noeud);
end Arbres_Binaires.De_Recherche.AVL;
