--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--    CORPS DU PAQUETAGE DES ARBRES BINAIRES DE RECHERCHE   --
--                         POINTES                          --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Ada.Tags;
with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits; 
package body Arbres_Binaires.De_Recherche is
-- procdures de parcours
  procedure Recherche_Locale (L_Arbre: in out Arbre_Rech; 
                              Sur: in Cle) is
    Le_Noeud: Pt_Noeud:= L_Arbre.Chemin(1).Le_Noeud;
  begin
    L_Arbre.Sommet := 1;
    while Le_Noeud /= null 
    and then Sur /= La_Cle (Noeud_Rech(Le_Noeud.all).Contenu) loop 
      Descendre(L_Arbre, 
        A_Gauche=> Sur < La_Cle(Noeud_Rech(Le_Noeud.all).Contenu));
      Le_Noeud := L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud;
    end loop;
    Trouve := Le_Noeud /= null;
  end Recherche_Locale;

  procedure Positionner(L_Arbre: in out Arbre_Rech; 
                         Sur: in Cle) is
  begin
    Recherche_Locale (L_Arbre, Sur);
    if Trouve then return; end if;
    L_Arbre.Sommet := 0;
    raise Erreur_Specification;             -- cl non trouve 
  end Positionner;

  function Contenu_Courant(L_Arbre: in Arbre_Rech)
                            return Element is
  begin 
    if A_La_Fin (L_Arbre) then raise Erreur_Specification; end if;
    return 
    Noeud_Rech(L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud.all).Contenu;
  end Contenu_Courant; 

  procedure Changer_Courant (L_Arbre : in out Arbre_Rech; 
                              Val : in Element) is
  begin
    if not A_La_Fin(L_Arbre) then
      declare
        V : Element renames Noeud_Rech
              (L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud.all).Contenu;
      begin
        if La_Cle(V) = La_Cle(Val) then 
          V := Val;
          return;
        end if;             -- changement de cl interdite
      end;
    end if;
    raise Erreur_Specification; 
  end Changer_Courant;

  procedure Ajouter (Dans : in out Arbre_Rech; 
                       Val : in Element) is 
    use Ada.Tags; 
  begin 
    if Arbre_Rech'Class(Dans)'Tag /= Arbre_Rech'Tag then
      raise Erreur_Specification;
    end if; 
    Recherche_Locale (Dans, Sur => La_Cle (Val)); 
    if Trouve then raise Erreur_Specification; end if; 
    Raccrocher(Dans, new Noeud_Rech'(Contenu => Val, 
                                      others => null)); 
  end Ajouter;

  procedure Supprimer(Dans:in out Arbre_Rech;
                        Niveau : out Positive) is
    Le_Noeud : Pt_Noeud;  -- noeud physiquement  supprimer
    Sauv_S: Positive;   -- endroits du chemin 
  begin
    if A_La_Fin(Dans) then raise Erreur_Specification; end if;
    Le_Noeud := Dans.Chemin(Dans.Sommet).Le_Noeud;
    if Le_Noeud.Gauche = null or Le_Noeud.Droite = null then 
      Niveau := Dans.Sommet; -- noeud physiquement supprim
      if Le_Noeud.Gauche = null then
        Raccrocher (Dans, Le_Noeud.Droite); 
      else 
        Raccrocher (Dans, Le_Noeud.Gauche);
      end if; 
      if Le_Noeud.Droite = null then
        Remonter_Bord_Droit (Dans);               -- ascendant
      else 
        Extremite_Bord_Gauche (Dans); -- descendant
      end if; 
    else --deux fils, recherche bord gauche du sous-arbre droit
                           -- qui est aussi la valeur suivante
      Sauv_S := Dans.Sommet;
      Descendre (Dans, A_Gauche => False);
      Extremite_Bord_Gauche (Dans);
      Le_Noeud := Dans.Chemin(Dans.Sommet).Le_Noeud;
      Noeud_Rech(Dans.Chemin(Sauv_S).Le_Noeud.all).Contenu := 
                                  Noeud_Rech(Le_Noeud.all).Contenu; 
      Raccrocher(Dans, Le_Noeud.Droite);
      Niveau := Dans.Sommet; -- noeud physiquement supprim
      Dans.Sommet := Sauv_S;
    end if;
    Liberer (Le_Noeud);         -- liberation de lespace occup
  end Supprimer;

  procedure Supprimer_Courant (Dans : in out Arbre_Rech) is 
    use Ada.Tags; 
    Bidon : Positive; 
  begin 
    if Arbre_Rech'Class(Dans)'Tag /= Arbre_Rech'Tag then
      raise Erreur_Specification;
    end if;
    Supprimer(Dans, Bidon); 
  end Supprimer_Courant;
end Arbres_Binaires.De_Recherche;
