--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--       PAQUETAGE DES ARBRES BINAIRES DE RECHERCHE         --
--                         POINTES                          --
--                                                          --
-- Note: les arbres binaires de recherche sont des arbres   --
--    <o,a,b> tels que:                                     --
--        a /= vide implique max(a) <= la_cle(o)            --
--        et b /= vide implique min(b) >= la_cle(o)         --
--    avec:                                                 --
--    max(<o,g,d>) = si d=vide alors la_cle(o) sinon max(d) --
--    min(<o,g,d>) = si g=vide alors la_cle(o) sinon min(g) --
--    le paquetage est un paquetage enfant de celui des     --
--        arbres binaires pointes, le type Arbre_Rech tant --
--        derive du type Arbre. Il herite donc des          --
--        operations  dexploration qui y sont definies. Il --
--        ajoute les operations suivantes.                  --
-- preconditions                                            --
--    Positionner (L_Arbre, Sur): Sur appartient a L_Arbre  --
--    Contenu_Courant(L_Arbre), Supprimer_Courant,          --
--        Supprimer: not A_La_Fin(L_Arbre)                  --
--    Changer_Courant(L_Arbre, Val): not A_La_Fin(L_Arbre)  --
--        et La_Cle(Val) = La_Cle(Contenu_Courant(L_Arbre)) --
--    Ajouter (Dans, Val): La_Cle(Val) nappartient pas a   --
--        Dans                                              --
-- proprietes fonctionnelles                                --
--    Positionner (L_Arbre, Sut) etablit le chemin vers le  --
--        noeud ayant pour cle Sur                          --
--    Contenu_Courant(L_Arbre) retourne la valeur du noeud  --
--        situe a lextremite du chemin actuel              --
--    Changer_Courant(L_Arbre, Val) change la valeur du     --
--        noeud situe a lextremite du chemin actuel        --
--    Ajouter (Dans, Val) ajoute un nouveau noeud dans      --
--        larbre, avec comme contenu Val                   --
--    Supprimer_Courant (L_Arbre) supprime le noeud courant --
-- operations de service                                    --
--    Recherche_locale (L_Arbre, Sur) etablit le chemin     --
--        ou doit se trouver un noeud de cle Sur, et met    --
--        a vrai la variable Trouve si le noeud existe      --
--    Supprimer (Dans, Niveau) supprime le noeud courant,   --
--        et retourne le Niveau du noeud physiquement       --
--        supprime sur le chemin                            --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

generic
  type Element is private;
  type Cle is private;
  with function La_Cle(E: in Element) return Cle is <>;
  with function "<"(U, V: in Cle) return Boolean is <>;
package Arbres_Binaires.De_Recherche is
  type Arbre_Rech is new Arbre with null record; 
  procedure Positionner (L_Arbre: in out Arbre_Rech; 
                          Sur: in Cle);
  function Contenu_Courant (L_Arbre: in Arbre_Rech)
                                              return Element;
  procedure Changer_Courant (L_Arbre : in out Arbre_Rech;
                              Val : in Element);
  procedure Ajouter (Dans : in out Arbre_Rech;
                       Val : in Element); 
  procedure Supprimer_Courant (Dans : in out Arbre_Rech);
private
  type Noeud_Rech is new Noeud with
    record
      Contenu : Element;
    end record; 
  Trouve : Boolean;
  procedure Recherche_Locale (L_Arbre: in out Arbre_Rech; 
                              Sur: in Cle); 
  procedure Supprimer (Dans : in out Arbre_Rech;
                        Niveau : out Positive);
end Arbres_Binaires.De_Recherche;
