--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--         CORPS DU PAQUETAGE DES ARBRES BINAIRES           --
--                         POINTES                          --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 27 mai 1997                       --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Arbres_Binaires is
-- procdures de parcours
  procedure Descendre (Repere : in out Position; 
                        P : in out Pt_Noeud; 
                        A_Gauche : in Boolean) is
  begin 
    Repere := (P, A_Gauche);
    if A_Gauche then P := P.Gauche;
    else P := P.Droite;
    end if;
  end Descendre;

  procedure Raccrocher(Repere: in Position; P: in Pt_Noeud) is
  begin 
    if Repere.A_Gauche then Repere.Le_Noeud.Gauche := P;
    else Repere.Le_Noeud.Droite := P;
    end if;
  end Raccrocher;

  procedure Descendre (Dans : in out Arbre; 
                        A_Gauche : in Boolean) is
    R : Position renames Dans.Chemin(Dans.Sommet);
    Le_Noeud : Pt_Noeud := R.Le_Noeud;
  begin 
    if Dans.Sommet = Dans.Hauteur_Max then 
      Dans.Sommet := 0;    -- positionnement indfini
      raise Limite_Implantation; 
    end if;
    R.A_Gauche := A_Gauche;
    Dans.Sommet := Dans.Sommet + 1;
    if A_Gauche then Le_Noeud := Le_Noeud.Gauche;
    else Le_Noeud := Le_Noeud.Droite;
    end if;
    Dans.Chemin(Dans.Sommet).Le_Noeud := Le_Noeud;
  end Descendre;

  procedure Raccrocher (Dans : in out Arbre; P : in Pt_Noeud; 
                          En : in Natural := 0) is 
    H : Natural := En;
  begin 
    if H = 0 then H := Dans.Sommet; end if;
    Dans.Chemin(H).Le_Noeud := P;
    if H > 1 then Raccrocher (Dans.Chemin(H - 1), P); end if;
  end Raccrocher;

  procedure Extremite_Bord_Gauche (L_Arbre: in out Arbre) is
  begin
    while L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud /= null loop
      Descendre (L_Arbre, A_Gauche => True);
    end loop;
    L_Arbre.Sommet := L_Arbre.Sommet - 1; -- on est alle trop loin
  end Extremite_Bord_Gauche;

  procedure Remonter_Bord_Droit (L_Arbre: in out Arbre) is
  begin
    loop                           -- remonte depuis droite
      L_Arbre.Sommet := L_Arbre.Sommet - 1; 
      exit when L_Arbre.Sommet = 0 
                 or else L_Arbre.Chemin(L_Arbre.Sommet).A_Gauche;
    end loop;
  end Remonter_Bord_Droit;

  procedure Premiere_Feuille (L_Arbre : in out Arbre) is
    Le_Noeud : Pt_Noeud;
  begin
    loop         -- recherche  partir du frre droit
      Extremite_Bord_Gauche (L_Arbre);
      Le_Noeud := L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud.Droite;
      exit when Le_Noeud = null;             -- c'est une feuille
      Descendre (L_Arbre, A_Gauche => False);
    end loop;
  end Premiere_Feuille;

  procedure Positionner_Sur_Premier (L_Arbre : in out Arbre) is 
  begin
    L_Arbre.Sommet := 1;
    Extremite_Bord_Gauche (L_Arbre); 
  end Positionner_Sur_Premier;

  procedure Passer_Au_Suivant (L_Arbre : in out Arbre) is
  begin
    if A_La_Fin(L_Arbre) then raise Erreur_Specification; end if;
    if L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud.Droite /= null then
      Descendre(L_Arbre, A_Gauche => False);
      Extremite_Bord_Gauche (L_Arbre); 
    else
      Remonter_Bord_Droit (L_Arbre);
    end if;
  end Passer_Au_Suivant;

  procedure Positionner_Sur_Premier_Prefixe 
                                        (L_Arbre : in out Arbre) is 
  begin
    if L_Arbre.Chemin(1).Le_Noeud /= null then 
      L_Arbre.Sommet := 1;
    else
      L_Arbre.Sommet := 0;
    end if;
  end Positionner_Sur_Premier_Prefixe;

  procedure Passer_Au_Suivant_Prefixe(L_Arbre : in out Arbre) is
    Le_Noeud : Pt_Noeud;
  begin
    if A_La_Fin(L_Arbre) then raise Erreur_Specification; end if;
    Le_Noeud := L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud;
    if Le_Noeud.Gauche /= null then
      Descendre (L_Arbre, A_Gauche => True);
    else 
      while Le_Noeud.Droite = null loop
        Remonter_Bord_Droit (L_Arbre);
        if L_Arbre.Sommet = 0 then return; end if;
        Le_Noeud := L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud;
      end loop;
      Descendre (L_Arbre, A_Gauche => False);
    end if;
  end Passer_Au_Suivant_Prefixe;

  procedure Positionner_Sur_Premier_Postfixe 
                                     (L_Arbre : in out Arbre) is 
  begin
    L_Arbre.Sommet := 0;
    if L_Arbre.Chemin(1).Le_Noeud = null then return; end if;
    L_Arbre.Sommet := 1;
    Premiere_Feuille (L_Arbre);
  end Positionner_Sur_Premier_Postfixe;

  procedure Passer_Au_Suivant_Postfixe (L_Arbre : in out Arbre) is
  begin
    if A_La_Fin(L_Arbre) then raise Erreur_Specification; end if;
    L_Arbre.Sommet := L_Arbre.Sommet - 1;
    if L_Arbre.Sommet /= 0 then 
      declare
        R : Position renames L_Arbre.Chemin(L_Arbre.Sommet);
      begin
        if R.A_Gauche and then R.Le_Noeud.Droite /= null then 
          Descendre (L_Arbre, A_Gauche => False);
          Premiere_Feuille (L_Arbre);
        end if;
      end;
    end if;
  end Passer_Au_Suivant_Postfixe;

  function A_La_Fin (L_Arbre: in Arbre) return Boolean is
  begin
    return L_Arbre.Sommet = 0;
  end A_La_Fin;

  function Hauteur_Courante(L_Arbre: in Arbre) return Natural is 
  begin 
    if A_La_Fin(L_Arbre) then raise Erreur_Specification; end if;
    return L_Arbre.Sommet - 1;
  end Hauteur_Courante;

  procedure Finalize (L_Arbre : in out Arbre) is
    Q : Pt_Noeud;
  begin       -- suppression dans lordre de la liste infixe
    Positionner_Sur_Premier (L_Arbre);
    while L_Arbre.Sommet /= 0 loop
      Q := L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud; 
      if Q.Droite = null then            -- extraire Q du chemin,
        L_Arbre.Sommet := L_Arbre.Sommet - 1; 
      else
        L_Arbre.Chemin(L_Arbre.Sommet).Le_Noeud := Q.Droite;
        Extremite_Bord_Gauche (L_Arbre); 
      end if;
      Liberer (Q);
    end loop;
    L_Arbre.Chemin(1).Le_Noeud := null;
  end Finalize; 
  procedure Vider (L_Arbre : in out Arbre) renames Finalize; 
end Arbres_Binaires;
