--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--          CORPS DE LA PROCEDURE D'ADJONCTION              --
--                DANS UN FICHIER BALANCE                   --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

procedure Balance_IO.Ajouter (File : in out File_Type; 
                               Val : in Element) is
  use N_B, IO;
  P : Pt_Descr_Bloc renames File.Bloc_Courant;
  Q : Pt_Descr_Bloc;
  V : Element := Val;                      -- valeur  insrer
  procedure Ajuster_Avec_Droite is
  begin
    if Q.Indice > Degre(Q.Contenu) + 1 then -- chemin passe  droite
      Q.Precedant.Indice := Q.Precedant.Indice + 1; 
      if P = Q then P := File.Frere; end if;
      File.Frere.Indice := Q.Indice - Degre(Q.Contenu) - 1; 
      Echanger_Avec_Frere (File, Q);
    elsif Q.Indice = Degre(Q.Contenu) + 1 and P = Q then 
      P := Q.Precedant; -- valeur passee au pere
    end if;
  end Ajuster_Avec_Droite;
  procedure Ajuster_Avec_Gauche is
  begin
    if Q.Indice > 1 then               -- toujours dans le mme nud
      Q.Indice := Q.Indice - 1;         -- qui a perdu une valeur
    else    -- le nud droit nest plus sur le chamin, cest la gauche
      Q.Precedant.Indice := Q.Precedant.Indice - 1;
      if P = Q then P := Q.Precedant; end if;
      Echanger_Avec_Frere (File, Q);
      Q.Indice := Degre(Q.Contenu) + 1;
    end if;
  end Ajuster_Avec_Gauche;
begin
  if Mode (File.File) = IO.In_File then raise Mode_Error; end if;
  Recherche_Chemin (File, La_Cle (Val));
  if Trouve then raise Cle_Existante; end if;       -- dj l
  Inserer (P.Contenu, P.Indice, V); -- adjonction feuille
  Q := P;
  while Degre(Q.Contenu) > Degre_Max loop 
                              -- le nud courant est trop gros
    if Q.Precedant = null then     -- il faut clater la racine
      Q.Num_Bloc := Allouer_Bloc (File);   -- plus racine
      if File.Frere = null then 
        File.Frere := new Descr_Bloc;
      end if;
      File.Frere.Num_Bloc := Q.Num_Bloc + 1;
      Eclater (Le_Noeud => Q.Contenu, En => Ordre,
               Val_Separ => V, Nouveau => File.Frere.Contenu);
      Ecrire (File, Q);    -- crire les deux nuds crs
      Ecrire (File, File.Frere);
      Q.Precedant := new Descr_Bloc;
      Q.Precedant.Suivant := Q;
      Initialiser_Noeud (Le_Noeud => Q.Precedant.Contenu,
                         Val_Separ => V,
                         Gauche => Q.Num_Bloc,
                         Droite => File.Frere.Num_Bloc);
      Q.Precedant.Num_Bloc := 1; -- nouvelle racine de l'arbre
      Q.Precedant.Indice := 1;
      Ajuster_Avec_Droite;
      Q := Q.Precedant;
      exit;
    end if;
               -- ce n'est pas la racine, on retrouve son pre
                    -- on essaie d'abord un transfert  gauche
    if Q.Precedant.Indice /= 1 then
      Lire_Frere (File, Q, -1);
      if Degre (File.Frere.Contenu) < Degre_Max then -- OK
        Transfert_Droite_Gauche (Le_Noeud_Pere=>Q.Precedant.Contenu, 
                                 Indice => Q.Precedant.Indice - 1,
                                 Gauche => File.Frere.Contenu, 
                                 Droite => Q.Contenu);
        Ecrire (File, File.Frere);
        Ecrire (File, Q);
        Ajuster_Avec_Gauche;
        Q := Q.Precedant;
        exit;
      end if;
    end if;
                    -- on essaie ensuite un transfert  droite
    if Q.Precedant.Indice <= Degre (Q.Precedant.Contenu) then
      Lire_Frere (File, Q, +1);
      if Degre (File.Frere.Contenu) < Degre_Max then -- OK
        Transfert_Gauche_Droite (Le_Noeud_Pere=>Q.Precedant.Contenu, 
                                 Indice => Q.Precedant.Indice,
                                 Gauche => Q.Contenu, 
                                 Droite => File.Frere.Contenu);
        Ecrire (File, Q);
        Ecrire (File, File.Frere);
        Ajuster_Avec_Droite;
        Q := Q.Precedant;
        exit;
      end if;
    end if;
                    -- aucun transfert, donc on clate le bloc
    if File.Frere = null then 
      File.Frere := new Descr_Bloc;
    end if;
    File.Frere.Num_Bloc := Allouer_Bloc (File);
    Eclater (Le_Noeud => Q.Contenu, En => Ordre,
             Val_Separ => V, Nouveau => File.Frere.Contenu);
    Ecrire (File, Q);
    Ecrire (File, File.Frere);
    Inserer (Q.Precedant.Contenu, Q.Precedant.Indice, 
             V, File.Frere.Num_Bloc);
    Ajuster_Avec_Droite;
    Q := Q.Precedant;
  end loop;
  Ecrire (File, Q); -- rcrire le dernier bloc modifi
end Balance_IO.Ajouter;
