--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--         CORPS DE LA PROCEDURE DE SUPPRESSION             --
--                 DANS UN FICHIER BALANCE                  --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

procedure Balance_IO.Supprimer_Courant (File: in out File_Type) is
  use N_B, IO;
  P : Pt_Descr_Bloc renames File.Bloc_Courant;
  Q : Pt_Descr_Bloc;
  R : Pt_Descr_Bloc;
  Au_Dela_Suivant : Boolean := False; -- permet de se rappeler que 
    -- lon est positionne apres valeur dans le noeud modifie
begin
  if Mode (File.File) = IO.In_File then raise Mode_Error; end if;
  if A_La_Fin (File) then raise End_Error; end if;
  Q := P;
  if not Est_Feuille (P.Contenu) then 
    Au_Dela_Suivant := True;
    P.Indice := P.Indice + 1;
    Bord_Gauche (File); -- recherche suprieur pour remplacement
    Changer_Valeur(Q.Contenu, Q.Indice - 1,
                   Ieme_Valeur (P.Contenu, P.Indice));
    Ecrire (File, Q);
  end if;
  Supprimer (P.Contenu, P.Indice); 
         -- Q pointe sur le nud contenant la valeur supprime
  while Degre (P.Contenu) < Ordre loop    
                             -- le nud courant est trop petit
    if P.Precedant = null then               -- c'est la racine
      if Degre(P.Contenu) = 0
      and then not Est_Feuille (P.Contenu) then
        R := P; -- il n'y a plus rien dedans
        P := P.Suivant;
        P.Precedant := null;
        P.Num_Bloc := 1;               -- nouvelle racine
        Liberer (R);
      end if;
      exit;
    end if;
                -- ce n'est pas la racine, on retrouve le pre
            -- on essaie d'abord un transfert depuis la droite
    if P.Precedant.Indice <= Degre (P.Precedant.Contenu) then
      Lire_Frere (File, P, +1);
      if Degre (File.Frere.Contenu) > Ordre then      -- OK
        Transfert_Droite_Gauche (Le_Noeud_Pere=>P.Precedant.Contenu,
                                 Indice => P.Precedant.Indice,
                                 Gauche => P.Contenu,
                                 Droite => File.Frere.Contenu);
        Ecrire (File, P);
        Ecrire (File, File.Frere);
        P := P.Precedant;
        exit; -- pas de changement du chemin ici
      end if;
    end if;
                        -- on essaie ensuite depuis la gauche
    if P.Precedant.Indice /= 1 then
      Lire_Frere (File, P, -1);
      if Degre (File.Frere.Contenu) > Ordre then      -- OK
        Transfert_Gauche_Droite (Le_Noeud_Pere=>P.Precedant.Contenu,
                                 Indice => P.Precedant.Indice - 1,
                                 Gauche => File.Frere.Contenu,
                                 Droite => P.Contenu);
        Ecrire (File, File.Frere);
        Ecrire (File, P);
        if P.Precedant = Q then -- ajustement chemin:
          Q := P;           -- la valeur suivante vient de descendue
          Au_Dela_Suivant := False; -- P.Indice repre cette valeur
        else 
          P.Indice := P.Indice + 1; -- valeurs et fils sont decales
        end if; 
        P := P.Precedant;
        exit;
      end if;
    -- transfert impossible, on regroupe avec celui de gauche
      P.Precedant.Indice := P.Precedant.Indice - 1;
      File.Frere.Indice := Degre(File.Frere.Contenu) + 1;
      Regrouper(Le_Noeud_Pere => P.Precedant.Contenu,
                Indice => P.Precedant.Indice,
                Gauche => File.Frere.Contenu,
                Droite => P.Contenu);
      Echanger_Avec_Frere (File, P);
      if P.Precedant = Q then -- ajustement du chemin:
        Q := P;          -- la valeurs suivante vient de descendre
        Au_Dela_Suivant := False; -- P.Indice repre cette valeur
      else 
        P.Indice := P.Indice + File.Frere.Indice; -- decalages
        if Q = File.Frere then Q := P; end if; -- changt bloc
      end if;
    else        -- pas de nud  gauche, et transfert 
                -- impossible depuis la droite, on regroupe 
                -- avec celui de droite, qui a deja t lu
      Regrouper(Le_Noeud_Pere => P.Precedant.Contenu,
                Indice => P.Precedant.Indice,
                Gauche => P.Contenu,
                Droite => File.Frere.Contenu);
       -- aucun ajustement de chemin
    end if;
    Ecrire (File, P);      -- le frre devient inutile
    P := P.Precedant;      -- pour continuer
  end loop;
  Ecrire (File, P);     -- rcriture du dernier bloc
  P := Q;
  if Au_Dela_Suivant then P.Indice := P.Indice - 1; -- retour suivant
  else Valeur_Suivante (File); -- controle interieur du nud 
  end if;
end Balance_IO.Supprimer_Courant;
