--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--       CORPS DU PAQUETAGE DES FICHIERS BALANCES           --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Balance_IO is
  use N_B, IO;
  Cor_Mode : constant array (File_Mode) of IO.File_Mode := 
                                 (IO.In_File, IO.Inout_File);

      -- echange le nud avec le frere dans la liste des blocs
  procedure Echanger_Avec_Frere (File: in out File_Type;
                         Noeud : in out Pt_Descr_Bloc) is
    R : Pt_Descr_Bloc := File.Frere;
  begin     
    R.Precedant := Noeud.Precedant;
    R.Suivant := Noeud.Suivant;
    R.Precedant.Suivant := R;
    if R.Suivant /= null then 
      R.Suivant.Precedant := R;
    end if;
    File.Frere := Noeud;
    Noeud := R;
  end Echanger_Avec_Frere;

          -- procdures internes pour lire ou ecrire les nuds
  procedure Lire_Racine (File : in out File_Type) is 
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    if P = null then
      P := new Descr_Bloc;
      P.Num_Bloc := 1;
      Read(File.File, P.Contenu, IO.Count'val(P.Num_Bloc));
      if not Longueurs_Compatibles (P.Contenu) then 
        raise Data_Error;
      end if;
    else 
      while P.Precedant /= null loop 
        P := P.Precedant;    -- retrouver le bloc racine 
      end loop;
    end if;
  end Lire_Racine;

  procedure Lire_Fils(File:in out File_Type) is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
    NB : Positive := Ieme_Fils (P.Contenu, P.Indice);
  begin 
    if P.Suivant = null then
      P.Suivant := new Descr_Bloc;
      P.Suivant.Precedant := P;
    end if;
    P := P.Suivant;    -- sur le bloc fils
    if P.Num_Bloc = NB then null;
    elsif File.Frere/=null and then File.Frere.Num_Bloc=NB then 
      Echanger_Avec_Frere (File, P);
    else 
      P.Num_Bloc := NB;
      Read (File.File, P.Contenu, IO.Count'val(P.Num_Bloc));
      if not Longueurs_Compatibles (P.Contenu) then 
        raise Data_Error;
      end if;
    end if;
  end Lire_Fils;

  procedure Lire_Frere (File : in out File_Type;
                         Noeud : in Pt_Descr_Bloc;
                         Depl : in Integer) is 
    P : Pt_Descr_Bloc renames File.Frere;
    NB : Positive := Ieme_Fils(Noeud.Precedant.Contenu, 
                               Noeud.Precedant.Indice + Depl);
  begin
    if P = null then
      P := new Descr_Bloc;
    end if;
    if P.Num_Bloc /= NB then 
      P.Num_Bloc := NB;
      Read (File.File, P.Contenu, IO.Count'val(P.Num_Bloc));
      if not Longueurs_Compatibles (P.Contenu) then 
        raise Data_Error;
      end if;
    end if;
  end Lire_Frere;

  procedure Ecrire (File : in File_Type; P : in Pt_Descr_Bloc) is
  begin
    Write (File.File, P.Contenu, IO.Count'val(P.Num_Bloc));
  end Ecrire;

  function Allouer_Bloc (File : in File_Type) return Positive is
  begin    -- numro du premier bloc aprs la fin du fichier
    return IO.Count'Pos (Size(File.File)) + 1;
  end Allouer_Bloc;

  procedure Liberer_Les_Blocs (File : in out File_Type) is
  begin 
    Liberer (File.Frere);
    if File.Bloc_Courant = null then return; end if;
    while File.Bloc_Courant.Precedant /= null loop
      File.Frere := File.Bloc_Courant.Precedant;
      File.Bloc_Courant.Precedant := File.Frere.Precedant;
      Liberer (File.Frere);
    end loop;
    while File.Bloc_Courant /= null loop 
      File.Frere := File.Bloc_Courant;
      File.Bloc_Courant := File.Bloc_Courant.Suivant;
      Liberer (File.Frere);
    end loop;
  end Liberer_Les_Blocs;

  procedure Bord_Gauche (File: in out File_Type) is
      -- recherche de la valeur la plus  gauche du sous-arbre
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    loop
      Lire_Fils (File);
      P.Indice := 1;
      exit when Est_Feuille (P.Contenu);
    end loop;
  end Bord_Gauche;

  procedure Valeur_Suivante (File: in out File_Type) is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin                        -- remonte sur un bord droit
    while P.Indice > Degre (P.Contenu) 
    and then P.Precedant /= null loop
      P := P.Precedant;
    end loop;
  end Valeur_Suivante;

  procedure Recherche_Chemin (File : in out File_Type; 
                               C : in Cle) is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
                -- procdure interne de recherche d'une valeur
  begin            -- et de construction du chemin
    Lire_Racine (File);
    loop
      P.Indice := Rang (P.Contenu, C);
      Trouve := P.Indice <= Degre (P.Contenu)
           and then C = La_Cle (Ieme_Valeur (P.Contenu,
                                              P.Indice));
      exit when Trouve or Est_Feuille (P.Contenu); 
      Lire_Fils (File);
    end loop;
  end Recherche_Chemin;
                                      -- procdures exportes
  procedure Creer (File : in out File_Type;
                    Mode : in File_Mode := Inout_File;
                    Name : in String := "") is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    Create (File.File, Cor_Mode(Mode), Name);
    P := new Descr_Bloc;
    P.Num_Bloc := 1;
    Ecrire (File, P);    -- cration bloc vide  la racine
    File.Frere := null;
  end Creer;

  procedure Ouvrir (File : in out File_Type;
                     Mode : in File_Mode := Inout_File;
                     Name : in String := "") is
  begin
    Open (File.File, Cor_Mode(Mode), Name);
    File.Bloc_Courant := null;
    File.Frere := null;
  end Ouvrir;

  procedure Fermer (File : in out File_Type) is
  begin 
    Liberer_Les_Blocs (File);
    Close (File.File);
  end Fermer;

  procedure Detruire (File : in out File_Type) is
  begin
    Liberer_Les_Blocs (File);
    Delete (File.File);
  end Detruire;

  procedure Positionner (File: in out File_Type; Sur: in CLE) is
  begin
    Recherche_Chemin (File, Sur);
    if not Trouve then 
      Valeur_Suivante (File); -- sur lment de cl suprieure
      raise Cle_Inconnue;
    end if;
  end Positionner;

  procedure Positionner_Sur_Premier (File : in out File_Type) is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    Lire_Racine (File);
    P.Indice := 1;
    if not Est_Feuille (P.Contenu) then 
      Bord_Gauche (File); 
    end if;
  end Positionner_Sur_Premier;

  procedure Passer_Au_Suivant (File : in out File_Type) is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    if A_La_Fin (File) then raise End_Error; end if;
    P.Indice := P.Indice + 1;
    if not Est_Feuille (P.Contenu) then 
      Bord_Gauche (File); 
    else 
      Valeur_Suivante (File);
    end if;
  end Passer_Au_Suivant;

  function A_La_Fin (File : in File_Type) return Boolean is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    return P = null or else P.Indice > Degre (P.Contenu); 
  end A_La_Fin;

  function Contenu_Courant (File : in File_Type) 
                                          return Element is
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    if A_La_Fin (File) then raise End_Error; end if;
    return Ieme_Valeur (P.Contenu, P.Indice); 
  end Contenu_Courant;

  procedure Changer_Courant (File : in File_Type; 
                              Val : in Element) is 
    P : Pt_Descr_Bloc renames File.Bloc_Courant;
  begin
    if Mode (File.File) = IO.In_File then 
      raise Mode_Error; 
    end if;
    if A_La_Fin (File) then raise End_Error; end if;
    if La_Cle (Val) /= 
           La_Cle (Ieme_Valeur (P.Contenu, P.Indice)) then 
      raise Cle_Changee; 
    end if;
    Changer_Valeur (P.Contenu, P.Indice, Val);
    Ecrire (File, P);
  end Changer_Courant;
end Balance_IO;
