--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--          CORPS DU PAQUETAGE FILE CIRCULAIRE              --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

package body File_Circulaire is
	type Place;
	type Pt_Place is access Place;
	type Place is
	  record
		Contenu : Element;
		Suivant : Pt_Place := null;
	  end record;
	Dernier : Pt_Place;

	procedure Vider is
	begin				 -- structuration en liste circulaire
	  Dernier.Suivant := Dernier;
	end Vider; 

	function Est_Vide return Boolean is
	begin
	  return Dernier.Suivant = Dernier;
	end Est_Vide;			  -- il n'y a que la tete fictive

	function Premier return Element is
	begin
	  if Est_Vide then raise File_Vide; end if;
	  return Dernier.Suivant.Suivant.Contenu;
					-- contenu de celui aprs la tete fictive
	end Premier;

	procedure Ajouter ( V : in Element) is
	  Nouveau : Pt_Place := new Place' (Contenu => V,
							     Suivant => Dernier.Suivant);
	  -- cration du nouveau, qui doit reprer la tte fictive
	begin
	  Dernier.Suivant := Nouveau;-- insertion aprs le dernier
	  Dernier := Nouveau;                   -- nouveau dernier
	end Ajouter;

	procedure Retirer is
	begin
	  if Est_Vide then raise File_Vide; end if;
	  Dernier.Suivant := Dernier.Suivant.Suivant;
      -- retrait de la liste circulaire de l'ancienne tte fictive,
	  -- l'ancienne vraie tte devient la nouvelle tte fictive
	end Retirer;

begin							 -- initialisation de la file
	Dernier := new Place;	   -- cration de la tte fictive
	Dernier.Suivant := Dernier;
						 -- structuration en liste circulaire
end File_Circulaire;
