--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--      CORPS DU PAQUETAGE DES FICHIERS DE HACHAGE          --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

package body Hachage_IO is
  Cor_Mode : constant array (File_Mode) of IO.File_Mode := 
								  (IO.In_File, IO.Inout_File);
  Trouve : Boolean;
  procedure Lire_Bloc(File: in out File_Type; NB: in Positive) is 
  begin
    if NB /= File.Num_Bloc then 
      File.Num_Bloc := NB;
      Read (File.File, File.Bloc, IO.Count'val(File.Num_Bloc));
    end if;
  end Lire_Bloc;

  procedure Recherche_Place (File : in out File_Type;
							  C : in Cle) is 
  begin
	File.Hachage_Courant := Espace'pos (Hachage (C))
							  - Espace'pos (Espace'first) + 1;
	Trouve := False;
    Lire_Bloc (File, File.Hachage_Courant);
    File.Indice := 1;
	loop
	  if File.Indice <= Longueur (File.Bloc.Contenu) then
		Trouve := La_Cle(Ieme(File.Bloc.Contenu, File.Indice)) = C;
		if Trouve then return; end if; -- arrt de la recherche
		File.Indice := File.Indice + 1;
	  elsif File.Bloc.Suivant = 0 then return; 
      else 
        Lire_Bloc (File, File.Bloc.Suivant);
        File.Indice := 1;
      end if;
	end loop;
  end Recherche_Place;

  procedure Valeur_Suivante (File : in out File_Type) is 
  begin
	while File.Indice > Longueur (File.Bloc.Contenu) loop
      if File.Bloc.Suivant /= 0 then 
        Lire_Bloc (File, File.Bloc.Suivant);
      else 
        File.Hachage_Courant := File.Hachage_Courant + 1;
        if File.Hachage_Courant = T_Num_B_Hach'Last then 
          return;
        end if;
        Lire_Bloc (File, File.Hachage_Courant);
      end if;
      File.Indice := 1;
	end loop;
  end Valeur_Suivante;

  procedure Creer (File : in out File_Type;
                    Mode : in File_Mode := Inout_File;
                    Name : in String := "") is
  begin
    Create (File.File, Cor_Mode (Mode), Name);
    Tronquer(File.Bloc.Contenu, 0); 
    File.Bloc.Suivant := 0; 
    for I in Espace loop       -- creation des premiers blocs pour 
      Write (File.File, File.Bloc); -- toutes les valeurs de hachage
    end loop;
    File.Hachage_Courant := T_Num_B_Hach'Last; -- fin du fichier
    File.Num_Bloc := 0;         -- pas de bloc present
  end Creer;

  procedure Ouvrir (File : in out File_Type;
                     Mode : in File_Mode := Inout_File;
                     Name : in String := "") is 
  begin
    Open (File.File, Cor_Mode(Mode), Name);
    if IO.Count'pos(Size(File.File)) = 0 then
      Tronquer(File.Bloc.Contenu, 0); -- fichier vide
      File.Bloc.Suivant := 0; 
      for I in Espace loop     -- creation des premiers blocs pour 
        Write (File.File, File.Bloc); -- les valeurs de hachage
      end loop;
    elsif IO.Count'pos(Size(File.File)) < T_Num_B_Hach'Last then 
      Close (File.File);  -- tous les premiers blocs nexistent pas
      raise Use_Error;
    end if;
    File.Hachage_Courant := T_Num_B_Hach'Last; -- fin du fichier
    File.Num_Bloc := 0;         -- pas de bloc present
  end Ouvrir;

  procedure Fermer (File : in out File_Type) is
  begin 
    Close (File.File);
  end Fermer;

  procedure Detruire (File : in out File_Type) is
  begin
    Delete (File.File);
  end Detruire;

  procedure Positionner (File: in out File_Type; Sur: in CLE) is
  begin
    Recherche_Place (File, Sur);
    if not Trouve then 
      Valeur_Suivante (File); -- sur lment suivant
      raise Cle_Inconnue;
    end if;
  end Positionner;

  procedure Positionner_Sur_Premier (File : in out File_Type) is
  begin
    File.Hachage_Courant := 1;
    Lire_Bloc (File, File.Hachage_Courant);
    File.Indice := 1;
    Valeur_Suivante (File); 
  end Positionner_Sur_Premier;

  procedure Passer_Au_Suivant (File : in out File_Type) is
  begin
    if A_La_Fin (File) then raise End_Error; end if;
    File.Indice := File.Indice + 1;
    Valeur_Suivante (File);
  end Passer_Au_Suivant;

  function A_La_Fin (File : in File_Type) return Boolean is
  begin
    return File.Hachage_Courant = T_Num_B_Hach'Last;
  end A_La_Fin;

  function Contenu_Courant (File: in File_Type) return Element is
  begin
    if A_La_Fin (File) then raise End_Error; end if;
    return Ieme (File.Bloc.Contenu, File.Indice); 
  end Contenu_Courant;

  procedure Changer_Courant (File : in out File_Type; 
                              Val : in Element) is 
  begin
    if Mode (File.File) = IO.In_File then 
      raise Mode_Error; 
    end if;
    if A_La_Fin (File) then raise End_Error; end if;
    if La_Cle (Val) /= 
           La_Cle (Ieme (File.Bloc.Contenu, File.Indice)) then 
      raise Cle_Changee; 
    end if;
    Changer_Ieme (File.Bloc.Contenu, File.Indice, Val);
    Write (File.File, File.Bloc, IO.Count'val(File.Num_Bloc));
  end Changer_Courant;

  procedure Ajouter (File: in out File_Type; Val: in Element) is
  begin
    if Mode (File.File) = IO.In_File then 
      raise Mode_Error; 
    end if;
	Recherche_Place (File, La_Cle (Val));
	if Trouve then raise Cle_Existante; end if;
	if File.Indice > Taille then -- plein, cration d'un nouveau
	  File.Bloc.Suivant:= IO.Count'pos(Size(File.File))+1;--chanage
	  Write (File.File, File.Bloc, IO.Count'val (File.Num_Bloc));
	  File.Num_Bloc := File.Bloc.Suivant;
	  File.Bloc.Suivant := 0;
	  Tronquer (File.Bloc.Contenu, 0);				 -- bloc vide
      File.Indice := 1;
	end if;
	Prolonger (Dans => File.Bloc.Contenu, Val => Val);
	Write (File.File, File.Bloc, IO.Count'val(File.Num_Bloc));
  end Ajouter;

  procedure Supprimer_Courant (File : in out File_Type) is
  begin
    if Mode (File.File) = IO.In_File then 
      raise Mode_Error; 
    end if;
    if A_La_Fin (File) then raise End_Error; end if;
	Supprimer (File.Bloc.Contenu, File.Indice);
	Write (File.File, File.Bloc, IO.Count'val(File.Num_Bloc));
    Valeur_Suivante (File);
  end Supprimer_Courant;
end Hachage_IO;
