--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--     CORPS DE LA PROCEDURE DE TRI DUNE LISTE CHAINEE     --
--                 METHODE DU TRI-FUSION                    --
--                                                          --
-- preconditions                                            --
--    coupe1 (l), coupe2 (l): longueur (l) >= 2             --
--    fusion (l1, l2): est-triee (l1) et est-triee (l2)     --
-- proprietes fonctionnelles                                --
--    longueur (coupe1 (l)) >0                              --
--    longueur (coupe2 (l)) >0                              --
--    est-permut(coupe1 (l) & coupe2 (l), l)                --
--    est_permut (fusion (l1, l2), l1 & l2)                 --
--    est-triee(fusion (l1, l2))                            --
--    tri-fusion (l) =                                      --
--        si longueur (l) < 2 alors l                       --
--        sinon fusion (tri-fusion (coupe1(l)),             --
--                      tri-fusion (coupe2(l)))             --
--        fsi                                               --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

procedure Listes_Chainees.Tri_Fus(La_Liste: in out Liste)is
                                  -- tri fusion
  function Fusion (P1, P2 : Pt_Place) return Pt_Place is
    Q1 : Pt_Place := P1;
    Q2 : Pt_Place := P2;
    Tete, Q3 : Pt_Place; 
  begin
    if Q1 = null then return Q2; end if;
    if Q2 = null then return Q1; end if;
    if Q1.Contenu < Q2.Contenu then
      Tete := Q1; Q1 := Q1.Suivant;
    else
      Tete := Q2; Q2 := Q2.Suivant;
    end if;
    Q3 := Tete;
    while Q1 /= null and Q2 /= null loop
      if Q1.Contenu < Q2.Contenu then
        Q3.Suivant := Q1;			 -- insrer au bout de L3
        Q1 := Q1.Suivant;				   -- supprimer de L1
      else
        Q3.Suivant := Q2;			 -- insrer au bout de L3
        Q2 := Q2.Suivant;				   -- supprimer de L2
      end if;
      Q3 := Q3.Suivant;					   -- avancer dans L3
    end loop;
    Q3.Suivant := Q1;			 -- on raccroche la fin de L1
    if Q2 /= null then
      Q3.Suivant := Q2;			 -- on raccroche la fin de L2
    end if;
    return Tete;	 -- on retourne la vritable tete
  end Fusion;

  procedure Couper (P : Pt_Place; P1, P2 : out Pt_Place) is
    Q : Pt_Place := P;
    Q1, Q2 : Pt_Place := null;
  begin
    Q1 := Q;						 -- on met le premier dans L1
    Q := Q.Suivant;
    Q2 := Q;							  -- et le second dans L2
    Q := Q.Suivant;
    P1 := Q1;
    P2 := Q2;
    while Q /= null loop
      Q1.Suivant := Q;						 -- au bout de L1
      Q1 := Q;
      Q := Q.Suivant;
      exit when Q = null;
      Q2.Suivant := Q;						 -- au bout de L2
      Q2 := Q;
      Q := Q.Suivant;
    end loop;
    Q1.Suivant := null;				  -- terminer la liste L1
    Q2.Suivant := null;				  -- terminer la liste L2
  end Couper;

  function Tri_Interne (P : Pt_Place) return Pt_Place is
      P1, P2 : Pt_Place;
  begin
    if P = null or else P.Suivant = null then
      return P;
    end if;
    Couper (P, P1, P2);
    return Fusion (Tri_Interne (P1), Tri_Interne (P2));
   end Tri_Interne;
begin
  La_Liste.Tete := Tri_Interne (La_Liste.Tete);
  La_Liste.Courant := La_Liste.Tete;
  La_Liste.Precedant := null;
end Listes_Chainees.Tri_Fus;
