--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--        CORPS DU PAQUETAGE DES LISTES CHAINEES            --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Listes_Chainees is

  function Longueur (La_Liste : in Liste) return Integer is
  begin
    return La_Liste.Longueur;
  end Longueur;

  function Contenu_Courant (La_Liste : in Liste) 
                                           return Element is
  begin
    if La_Liste.Courant = null then 
      raise Erreur_Specification; 
    end if;
    return La_Liste.Courant.Contenu;
  end Contenu_Courant;

  procedure Changer_Courant(La_Liste:in out Liste;
                             Val : in Element) is 
  begin
    if La_Liste.Courant = null then 
      raise Erreur_Specification; 
    end if;
    La_Liste.Courant.Contenu := Val;
  end Changer_Courant;

  procedure Positionner_Sur_Premier (La_Liste : in out Liste) is
  begin
    La_Liste.Courant := La_Liste.Tete;
    La_Liste.Precedant := null;
  end Positionner_Sur_Premier;

  procedure Passer_Au_Suivant (La_Liste : in out Liste) is
  begin
    if La_Liste.Courant = null then 
      raise Erreur_Specification; 
    end if;
    La_Liste.Precedant := La_Liste.Courant;
    La_Liste.Courant := La_Liste.Courant.Suivant;
  end Passer_Au_Suivant;

  function A_La_Fin (La_Liste : in Liste) return Boolean is
  begin
    return La_Liste.Courant = null;
  end A_La_Fin;

  procedure Inserer_Avant_Courant (La_Liste : in out Liste;
                                    Val : in Element) is
    Ajout : Pt_Place := new Place'(Contenu => Val,
                              Suivant => La_Liste.Courant);
  begin
    if La_Liste.Precedant = null then 
      La_Liste.Tete := Ajout;          -- au debut de la liste
    else
      La_Liste.Precedant.Suivant := Ajout;
    end if;
    La_Liste.Precedant := Ajout;
    La_Liste.Longueur := La_Liste.Longueur + 1;
  end Inserer_Avant_Courant;

  procedure Supprimer_Courant (La_Liste : in out Liste) is
    P : Pt_place := La_Liste.Courant;
  begin
    if La_Liste.Courant = null then 
      raise Erreur_Specification; 
    end if;
    La_Liste.Courant := La_Liste.Courant.Suivant;
    if La_Liste.Precedant = null then --au dbut de la liste
      La_Liste.Tete := La_Liste.Courant;
    else
      La_Liste.Precedant.Suivant := La_Liste.Courant;
    end if;
    La_Liste.Longueur := La_Liste.Longueur - 1;
    Liberer (P);
  end Supprimer_Courant;

  procedure Vider (La_Liste : in out Liste) is
    P : Pt_Place;
  begin
    while La_Liste.Tete /= null loop
      P := La_Liste.Tete;
      La_Liste.Tete := La_Liste.Tete.Suivant;
      Liberer (P);
    end loop;
    La_Liste.Longueur := 0;
    La_Liste.Precedant := null;
    La_Liste.Courant := null;
  end Vider;
end Listes_Chainees;
