--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--             PAQUETAGE DES LISTES CHAINEES                --
--                       POINTEES                           --
--                                                          --
-- Note: une liste pointee peut etre vue comme deux listes  --
--    celle des elements avant l'element courant, et        --
--    celle qui commence avec lelement courant             --
-- preconditions                                            --
--    Contenu_Courant(La_Liste), Changer_Courant(La_Liste), --
--    Passer_Au_Suivant (La_Liste),                         --
--    Supprimer_Courant (La_Liste):                         --
--        not A_La_Fin (La_Liste)                           --
-- proprietes fonctionnelles                                --
--    longueur (l) =                                        --
--        si l = creation alors 0                           --
--        sinsi l = inserer_avant_courant(l', e) alors      --
--            longueur(l')+1                                --
--        sinon soit l = supprimer_courant(l');             --
--            longueur(l')-1                                --
--        fsi                                               --
--    contenu_courant (<l1, e :: l2>) = e                   --
--    changer_courant (<l1, e' :: l2>, e) = <l1, e :: l2>,  --
--    positionner_sur_premier (<l1, l2>) = <lvide, l1 & l2> --
--    passer_au_suivant (<l1, e :: l2>) = <l1 & [e], l2>    --
--    a_la_fin (<l1, l2>) = estvide (l2)                    --
--    inserer_avant_courant (<l1, l2>, e) = <l1 & [e], l2>  --
--    supprimer_courant (<l1, e :: l2>) = <l1, l2>          --
--    vider (<l1, l2>) = <lvide, lvide>                     --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Ada.Finalization;
with Ada.Unchecked_deallocation;
generic
  type Element is private;
package Listes_Chainees is
  type Liste is limited private;
  function Longueur (La_Liste : in Liste) return Integer;
  function Contenu_Courant(La_Liste:in Liste)return Element;
  procedure Changer_Courant(La_Liste:in out Liste;
                             Val : in Element);
  procedure Positionner_Sur_Premier (La_Liste : in out Liste);
  procedure Passer_Au_Suivant (La_Liste : in out Liste);
  function A_La_Fin (La_Liste : in Liste) return Boolean;
  procedure Inserer_Avant_Courant (La_Liste : in out Liste;
                                    Val : in Element);
  procedure Supprimer_Courant (La_Liste : in out Liste);
  procedure Vider (La_Liste : in out Liste);
private
  type Place;
  type Pt_Place is access Place;
  type Place is
    record
      Contenu : Element;
      Suivant : Pt_Place := null;
    end record;
  type Liste is new Ada.Finalization.Limited_Controlled with
    record
      Longueur : Natural := 0;
      Tete, Precedant, Courant : Pt_Place := null;
    end record;
  procedure Liberer is new 
                  Ada.Unchecked_deallocation (Place, Pt_Place);
  procedure Finalize(La_Liste: in out Liste) renames Vider;
end Listes_Chainees;
