--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--      CORPS DU PAQUETAGE DES LISTES CONTIGUES EN TAS      --
--        enfant du PAQUETAGE DES LISTES CONTIGUES          --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Listes_Contigues.En_Tas is

  procedure Ordonner_Vers_Le_Bas (La_Liste: in out Liste; 
                                   En, Max : Positive) is
    U : Element := La_Liste.Ieme (En);
    subtype Indice is Positive range La_Liste.Ieme'Range;
    Le_Fils : Indice;                        -- indice du fils
    Le_Pere : Indice := En;                    -- indice du pre
  begin
    while 2 * Le_Pere <= Max loop            -- Il y a un fils
      Le_Fils := 2 * Le_Pere;    -- Choisir le plus petit fils
      if Le_Fils < Max and then             -- il a un frre
       La_Liste.Ieme(Le_Fils+1) < La_Liste.Ieme(Le_Fils) then
        Le_Fils := Le_Fils + 1;            -- qui est plus grand
      end if;
      exit when U < La_Liste.Ieme (Le_Fils);    -- bien plac
      La_Liste.Ieme (Le_Pere) := La_Liste.Ieme (Le_Fils);
      Le_Pere := Le_Fils;
                  -- remonter le fils et poursuivre en dessous
    end loop;
    La_Liste.Ieme (Le_Pere) := U;              -- place finale
  end Ordonner_Vers_Le_Bas;

  procedure Ordonner_Vers_Le_Haut (La_Liste: in out Liste; 
                                    En: Positive) is
    U : Element := La_Liste.Ieme (En);
    subtype Indice is Positive range La_Liste.Ieme'Range;
    Le_Fils : Indice := En;                 -- indice du fils
    Le_Pere : Indice;                       -- indice du pre
  begin
    while Le_Fils /= 1 loop                   -- le pere existe
      Le_Pere := Le_Fils / 2;
      exit when La_Liste.Ieme (Le_Pere) < U; -- place trouvee
      La_Liste.Ieme (Le_Fils) := La_Liste.Ieme (Le_Pere); 
                                            -- descendre le pere
      Le_Fils := Le_Pere;             -- on remonte dans l'arbre
    end loop;
    La_Liste.Ieme (Le_Fils) := U;              -- place finale
  end Ordonner_Vers_Le_Haut;

  procedure Ordonner_Le_Tas (La_Liste: in out Liste; 
                              Jusque : in Positive) is
  begin
    if Jusque > La_Liste.Longueur then 
      raise Erreur_Specification; 
    end if;
    for K in reverse 1..Jusque/2 loop
      Ordonner_Vers_Le_Bas (La_Liste, K, Jusque);
    end loop;
  end Ordonner_Le_Tas;

  function Min_Du_Tas (La_Liste: in Liste) return Element is
  begin
    if La_Liste.Longueur = 0 then 
      raise Erreur_Specification; 
    end if;
    return La_Liste.Ieme(1);
  end Min_Du_Tas;

  procedure Ajouter_Au_Tas (La_Liste: in out Liste; 
                             Val: in Element) is
  begin
    Prolonger (La_Liste, Val);
    Ordonner_Vers_Le_Haut (La_Liste, La_Liste.Longueur);
  end Ajouter_Au_Tas;

  procedure Supprimer_Du_Tas (La_Liste: in out Liste) is
  begin
    if La_Liste.Longueur = 0 then 
      raise Erreur_Specification; 
    end if;
    La_Liste.Ieme(1) := La_Liste.Ieme(La_Liste.Longueur);
    La_Liste.Longueur := La_Liste.Longueur - 1;
    if La_Liste.Longueur > 1 then
      Ordonner_Vers_Le_Bas (La_Liste, 1, La_Liste.Longueur);
    end if;
  end Supprimer_Du_Tas;

  procedure Ajuster_Le_Tas (La_Liste: in out Liste; 
                             En, Jusque : in Positive) is
  begin
    if En > Jusque or Jusque > La_Liste.Longueur then 
      raise Erreur_Specification; 
    end if;
    if 2 * En <= Jusque then 
      Ordonner_Vers_Le_Bas (La_Liste, En, Jusque); 
                                    -- ajustement vers les fils
    end if;
    if En > 1 then 
      Ordonner_Vers_Le_Haut (La_Liste, En); 
                                    -- ajustement vers le pere
    end if;
  end Ajuster_Le_Tas;
end Listes_Contigues.En_Tas;
