--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--         PAQUETAGE DES LISTES CONTIGUES EN TAS            --
--        enfant du PAQUETAGE DES LISTES CONTIGUES          --
--                                                          --
-- Note: la liste est la representation d'un arbre parfait  --
--    Ordonner_Le_Tas le rend partiellement ordonne         --
--    Les autres operations permettent lajout ou           --
--        lextraction des elements en conservant la        --
--        propriete partiellement ordonne                   --
--    Certaines operations supposent que la liste est deja  --
--        ordonnee en tas, mais ne le verifient pas         --
-- preconditions                                            --
--    Ordonner_Le_Tas(La_Liste, Jusque):                    --
--        1 <= Jusque <= Longueur(La_Liste)                 --
--    Min_Du_Tas(La_Liste), Supprimer_Du_Tas(La_Liste):     --
--        Longueur(La-Liste) /= 0 et est-tas(La_Liste)      --
--    Ajuster_Le_Tas(La_Liste,En,Jusque):                   --
--        1 <= En <= Jusque <= Longueur(La_Liste)           --
--        et il existe v telle que                          --
--          est-tas(changer-ieme(La_Liste(1..Jusque),En,v)) --
--        i.e. cest un tas sauf eventuellement en En       --
-- contrainte dimplantation                                --
--    Ajouter_Au_Tas(La_Liste, Val):                        --
--        Longueur (La_Liste) < La_Liste.Taille_Max         --
-- proprietes fonctionnelles                                --
--    est-tas(La_Liste) =                                   --
--        pour tout 2<=i<=Longueur(La_Liste):               --
--            Ieme(La_Liste,i/2) <= Ieme(La_Liste,i)        --
--    Ordonner_Le_Tas(La_Liste, Jusque) = l1 & l2 ou`       --
--        est-permut(l1,La_Liste(1..Jusque)) et est-tas(l1) --
--        et l2 = La_Liste(Jusque+1..Longueur(La_Liste))    --
--    Min_Du_Tas(La_Liste) = Ieme(La_Liste, 1)              --
--    Ajouter_Au_Tas(La_Liste, Val) = l telle que:          --
--        est-permut(l, prolonger(La_Liste), Val))          --
--        et est-tas(l)                                     --
--    Supprimer_Du_Tas(La_Liste) = l telle que:             --
--        est-permut(l,supprimer(La_Liste,1)) et est-tas(l) --
--    Ajuster_Le_Tas(La_Liste, En, Jusque) = ll & l2 ou`    --
--        est-permut(l1,La_Liste(1..Jusque)) et est-tas(l1) --
--        et l2 = La_Liste(Jusque+1..Longueur(La_Liste))    --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

generic
  with function "<"(U, V : Element) return Boolean is <>;
package Listes_Contigues.En_Tas is
  procedure Ordonner_Le_Tas (La_Liste: in out Liste; 
                              Jusque : in Positive);
           -- transforme le debut de la liste en tas
  function Min_Du_Tas (La_Liste: in Liste) return Element;
  procedure Ajouter_Au_Tas (La_Liste: in out Liste; 
                             Val: in Element);
  procedure Supprimer_Du_Tas (La_Liste: in out Liste);
  procedure Ajuster_Le_Tas (La_Liste: in out Liste; 
                             En, Jusque : in Positive); 
           -- deplace lelement En pour obtenir un tas
end Listes_Contigues.En_Tas;
