--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--                CORPS DE LA PROCEDURE DE                  --
--                TRI DUNE LISTE CONTIGUE                  --
--                 (methode du tri rapide)                  --
--                                                          --
-- preconditions                                            --
--    plus-petits(l), plus-grands(l), pivot(l):             --
--        non estvide(l)                                    --
-- propriete fonctionnelle                                  --
--    tri-rapide(l) =                                       --
--        si l = listevide alors listevide                  --
--        sinon tri-rapide(plus-petits(l)) & [pivot(l)] &   --
--                           tri-rapide(plus-grands(l)) fsi --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Pile;
procedure Listes_Contigues.Tri_Liste(La_Liste: in out Liste)is
  subtype Indice is Positive range La_Liste.Ieme'Range;
  type Intervalle is
    record
       Debut, Fin : Indice;
    end record;
  Cour : Intervalle;--dlimite la liste en cours de traitement
  Debut : Indice renames Cour.Debut;
  Fin : Indice renames Cour.Fin;
  Presumee : Indice;   -- place prsume o sera la valeur pivot

  package Pile_Intervalle is new Pile(Taille => Indice'size,
                                        Element => Intervalle);
  use Pile_Intervalle; 

  procedure Echanger (I, J : Indice) is
    U : Element := La_Liste.Ieme (J);
  begin
    La_Liste.Ieme (J) := La_Liste.Ieme (I);
    La_Liste.Ieme (I) := U;
  end Echanger;

  procedure Repartition is
    -- des lments entre Debut et Fin compris, par rapport 
    --  une valeur mdiane. Suppose Fin - Debut >= 2
    Pivot : Element;                      -- la valeur du pivot
    I, J : Indice;
  begin
    Presumee := (Debut + Fin) / 2;   -- place prsume a priori
                        -- choix parmi premier, milieu, dernier
    I := Debut;
    if La_Liste.Ieme (Presumee) < La_Liste.Ieme (I) then 
      I := Presumee; 
    end if;
    if La_Liste.Ieme (Fin) < La_Liste.Ieme (I) then 
      I := Fin; 
    end if;
    if Debut /= I then                -- I repre le plus petit
      Echanger (Debut, I);            -- le plus petit au dbut
    end if;
    if La_Liste.Ieme (Presumee) < La_Liste.Ieme (Fin) then
      Pivot := La_Liste.Ieme (Presumee);      -- celle du milieu
    else
      Pivot := La_Liste.Ieme (Fin);               -- la dernire
      La_Liste.Ieme (Fin) := La_Liste.Ieme (Presumee);
    end if;

     -- La_Liste.Ieme (Debut) <= Pivot <= La_Liste.Ieme (Fin)
     -- Presumee est une place vacante destine  recevoir Pivot
    I := Debut + 1;            -- placement des autres valeurs
    J := Fin - 1;
    while I < J loop
        -- Presumee est une place vacante, et I <= Presumee <= J
      while I<Presumee 
             and then not(Pivot < La_Liste.Ieme(I)) loop
        I := I + 1;                               -- bien place
      end loop;
      while J>Presumee 
             and then not(La_Liste.Ieme(J) < Pivot) loop
         J := J - 1;                           -- bien place
      end loop;
      exit when I = J;     -- tout est plac vis  vis du Pivot 
      if I = Presumee then -- cas 3
        La_Liste.Ieme (I) := La_Liste.Ieme (J);      -- dplacer
        Presumee := J;                 -- nouvelle place vacante
        I := I + 1;
      elsif J = Presumee then -- cas 2
        La_Liste.Ieme (J) := La_Liste.Ieme (I);      -- dplacer
        Presumee := I;                 -- nouvelle place vacante
        J := J - 1;
      else -- cas 1
        Echanger (I, J);                    -- dplacer les deux
        I := I + 1;
        J := J - 1;
      end if;
    end loop;
    La_Liste.Ieme (Presumee) := Pivot;        -- ranger le pivot
  end Repartition;

begin                       -- corps de la procdure principale
  if La_Liste.Longueur < 2 then return; end if;-- rien  faire
  Debut := 1;
  Fin := La_Liste.Longueur;
  loop
    while Fin - Debut > 1 loop
      Repartition;                   -- couper la liste en deux
      if Presumee-Debut < Fin-Presumee then -- + petite d'abord
        Empiler(Intervalle'(Debut=> Presumee + 1, Fin=> Fin));
        Fin := Presumee - 1;            -- c'est celle du dbut
      else
        Empiler(Intervalle'(Debut=> Debut, Fin=> Presumee-1));
        Debut := Presumee + 1;          -- c'est celle de la fin
      end if;
    end loop;
                             -- la liste est suffisamment petite
    if Fin = Debut + 1
    and then La_Liste.Ieme(Fin) < La_Liste.Ieme(Debut) then
      Echanger (Debut, Fin);
    end if;
    exit when Est_Vide;        -- toutes les listes sont faites
    Cour := Sommet;                            -- liste suivante
    Depiler;
  end loop;
end Listes_Contigues.Tri_Liste;
