--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--                CORPS DE LA PROCEDURE DE                  --
--                TRI DUNE LISTE CONTIGUE                  --
--                (methode du tri par tas)                  --
--                                                          --
-- Note: la liste est la representation d'un arbre parfait  --
--    premiere etape: on le rend partiellement ordonne      --
--    deuxieme etape: on extrait les elements en conservant --
--        la propriete partiellement ordonne                --
-- preconditions                                            --
--    ordonner (a):                                         --
--    a /= vide implique (g(a) = vide implique d(a) = vide) --
-- proprietes fonctionnelles                                --
--    ordonner(a) =                                         --
--        si a = vide ou a = <o, vide, vide> alors a        --
--        sinon soit a = <o, <o1, gg, dd>, d>;              --
--            si o>=o1 et (d=vide ou o>=racine(d)) alors a  --
--            sinsi o<o1 et (d=vide ou o1>=racine(d)) alors --
--                <o1, ordonner(<o, gg, dd>), d>            --
--            sinon soit a = <o, g, <o2, dg, dd>>;          --
--                <o2, g, ordonner(<o, dg, dd>)>            --
--    tri-sel-iter(l1, l) =                                 --
--        si l = listevide alors l1                         --
--        sinon tri-sel-iter(max(l)::l1, supprimer-max(l))  --
--        fsi                                               --
--    tri (l) = tri-sel-iter(listevide, l)                  --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

procedure Listes_Contigues.Tri_Liste(La_Liste: in out Liste)is
  subtype Indice is Positive range La_Liste.Ieme'Range;
  U : Element;          -- valeur de la racine lorsqu'on ordonne
  procedure Ordonner (En, Max : Indice) is
    Le_Fils : Indice;                        -- indice du fils
    Le_Pere : Indice := En;                    -- indice du pre
  begin
    while 2 * Le_Pere <= Max loop            -- Il y a un fils
      Le_Fils := 2 * Le_Pere;    -- Choisir le plus grand fils
      if Le_Fils < Max and then             -- il a un frre
       La_Liste.Ieme(Le_Fils) < La_Liste.Ieme(Le_Fils+1) then
        Le_Fils := Le_Fils + 1;            -- qui est plus grand
      end if;
      exit when La_Liste.Ieme (Le_Fils) < U;    -- bien plac
      La_Liste.Ieme (Le_Pere) := La_Liste.Ieme (Le_Fils);
      Le_Pere := Le_Fils;
                  -- remonter le fils et poursuivre en dessous
    end loop;
    La_Liste.Ieme (Le_Pere) := U;              -- place finale
  end Ordonner;
begin
                                       -- construction du tas
  for K in reverse Indice range 1..La_Liste.Longueur/2 loop
    U := La_Liste.Ieme (K);
    Ordonner (K, La_Liste.Longueur);
  end loop;
                                         -- extraction du tas
  for K in reverse Indice range 2 .. La_Liste.Longueur loop
    U := La_Liste.Ieme (K);                  -- dernire feuille
    La_Liste.Ieme (K) := La_Liste.Ieme (1); 
                             -- le plus grand dans le rsultat
    Ordonner (1, K - 1);
              -- ordonner avec la dernire feuille  la racine
  end loop;
end Listes_Contigues.Tri_Liste;
