--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--        CORPS DU PAQUETAGE DES LISTES CONTIGUES           --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Listes_Contigues is             -- le corps
  function Longueur (De : in Liste) return Natural is
  begin
    return De.Longueur;
  end Longueur;

  procedure Inserer (Dans : in out Liste; 
                      En : in Positive;
                      Val : in Element) is
  begin
    if En > Dans.Longueur + 1 then 
      raise Erreur_Specification; 
    end if;
    if Dans.Longueur = Dans.Taille_Max then 
      raise Limite_Implantation; 
    end if;
    Dans.Longueur := Dans.Longueur + 1;
    Dans.Ieme (En + 1..Dans.Longueur) :=
                            Dans.Ieme (En..Dans.Longueur - 1);
    Dans.Ieme (En) := Val;
  end Inserer;

  procedure Supprimer (Dans : in out Liste; 
                        En : in Positive) is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    Dans.Longueur := Dans.Longueur - 1;
    Dans.Ieme (En..Dans.Longueur) :=
                        Dans.Ieme (En + 1..Dans.Longueur + 1);
  end Supprimer;

  function Ieme (Dans : in Liste; 
                  En : in Positive) return Element is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    return Dans.Ieme (En);
  end Ieme;

  procedure Changer_Ieme (Dans : in out ListE; 
                           En : in Positive;
                           Val : in Element) is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    Dans.Ieme (En) := Val;
  end Changer_Ieme;

  procedure Prolonger (Dans : in out Liste; 
                        Val : in Element) is
  begin
    if Dans.Longueur = Dans.Taille_Max then 
      raise Limite_Implantation; 
    end if;
    Dans.Longueur := Dans.Longueur + 1;
    Dans.Ieme (Dans.Longueur) := Val;
  end Prolonger;

  procedure Tronquer (Dans : in out Liste; 
                       En : in Natural) is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    Dans.Longueur := En;
  end Tronquer;

  procedure Copier (Dans : in out Liste; 
                     La_Liste : in Liste) is
  begin
    if Dans.Taille_Max < La_Liste.Longueur then
      raise Erreur_Specification;
    end if;
    Dans.Longueur := La_Liste.Longueur;
    Dans.Ieme (1..La_Liste.Longueur) := 
                         La_Liste.Ieme (1..La_Liste.Longueur);
  end Copier;

  function "=" (Left, Right : in Liste) return Boolean is
  begin
    return Left.Longueur = Right.Longueur
       and then Left.Ieme (1..Left.Longueur) = 
                               Right.Ieme (1..Right.Longueur);
  end "="; 

end Listes_Contigues;
