--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--             PAQUETAGE DES LISTES CONTIGUES               --
--                                                          --
-- preconditions                                            --
--    Inserer(Dans, En, Val): 1 <= En <= Longueur(Dans) + 1 --
--    Supprimer (Dans, En), Ieme (Dans, En),                --
--    Changer_Ieme (Dans, En, Val), Tronquer (Dans, En):    --
--        1 <= En <= Longueur (Dans)                        --
-- contrainte dimplantation                                --
--    Inserer (Dans, En, Val), Prolonger (Dans, Val):       --
--        Longueur (Dans) < Dans.Taille_Max                 --
-- proprietes fonctionnelles                                --
--    longueur (l) =                                        --
--        si l = creation alors 0                           --
--        sinsi l = inserer(l', k, e) alors longueur(l')+1  --
--        sinon soit l = supprimer(l', k); longueur(l')-1   --
--        fsi                                               --
--    ieme(insrer(l, k, e), i) =                           --
--        si 1  i < k alors ieme(l, i)                     --
--        sinsi i = k alors e                               --
--        sinon ieme(l, i - 1)                              --
--        fsi                                               --
--    ieme(supprimer(l, k), i) =                            --
--        si 1  i < k alors ieme(l, i)                     --
--        sinon ieme(l, i + 1)                              --
--        fsi                                               --
--    changer_ieme (l, k, e) = inserer(supprimer(l,k),k,e)  --
--    prolonger (l, e) = inserer(l, longueur(l), e)         --
--    tronquer (l, k) =                                     --
--        si longueur(l) = k alors l                        --
--        sinon tronquer(supprimer(l,k),k) fsi              --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

generic
  type Element is private; -- type des lments dune liste
package Listes_Contigues is
  type Liste (Taille_Max : Positive) is private;
                                      -- le type liste fourni
  function Longueur (De : in Liste) return Natural;
  procedure Inserer (Dans : in out Liste;
                      En : in Positive;
                      Val : in Element);
  procedure Supprimer (Dans : in out Liste;
                        En : in Positive);
  function Ieme (Dans : in Liste;
                  En : in Positive) return Element;
            -- suppression suivi d'insertion au mme endroit
  procedure Changer_Ieme (Dans : in out Liste;
                           En : in Positive;
                           Val : in Element);
                   -- abrviation de insertion en longueur + 1
  procedure Prolonger (Dans : in out Liste;
                        Val : in Element);
    -- abrviation de suppression de tous ceux situs aprs En
  procedure Tronquer (Dans : in out Liste;
                       En : in Natural);
  procedure Copier (Dans : in out Liste; La_Liste : in Liste);
  function "=" (Left, Right : in Liste) return Boolean;
private
  type Table_Element is array (Positive range<>)of Element;
  type Liste (Taille_Max : Positive) is
    record   -- reprsentation effective du type liste fourni
      Longueur : Natural := 0;  --initialisation  la cration
      Ieme : Table_Element(1..Taille_Max);
    end record;
end Listes_Contigues;
