--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--     CORPS DU PAQUETAGE DES LISTES CONTIGUES ORDONNEES    --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Recherche_Contigue;
with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Listes_Contigues_Ordonnees is
  function "<" (U, V : in Element) return Boolean is
  begin                  -- fonction locale de simplification
    return La_Cle (U) < La_Cle (V);
  end "<";

 function Longueur (De : in Liste) return Natural is
  begin
    return De.Longueur;
  end Longueur;

  function Ieme (Dans : in Liste; 
                  En : in Positive) return Element is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    return Dans.Ieme (En);
  end Ieme;

  procedure Changer_Ieme (Dans : in out Liste; 
                           En : in Positive;
                           Val : in Element) is
  begin
    if En > Dans.Longueur 
    or else (En > 1 and then Val < Ieme(Dans, En - 1))
    or else (En < Dans.Longueur 
              and then Ieme(Dans, En + 1) < Val) then
      raise Erreur_Specification; -- Val pas a la bonne place
    end if;
    Dans.Ieme (En) := Val;
  end Changer_Ieme;

  procedure Inserer (Dans : in out Liste; 
                      En : in Positive;
                      Val : in Element) is
  begin
    if En > Dans.Longueur + 1 
    or else (En > 1 and then Val < Ieme(Dans, En - 1))
    or else (En < Dans.Longueur 
          and then Ieme(Dans, En + 1) < Val) then
      raise Erreur_Specification; -- Val pas a la bonne place
    end if;
    if Dans.Longueur = Dans.Taille_Max then 
      raise Limite_Implantation; 
    end if;
    Dans.Longueur := Dans.Longueur + 1;
    Dans.Ieme (En + 1..Dans.Longueur) :=
                            Dans.Ieme (En..Dans.Longueur - 1);
    Dans.Ieme (En) := Val;
  end Inserer;

  procedure Prolonger (Dans : in out Liste; 
                        Val : in Element) is
  begin
    if Dans.Longueur >= 1 
    and then Val < Ieme(Dans, Dans.Longueur) then
      raise Erreur_Specification;
    end if;
    if Dans.Longueur = Dans.Taille_Max then 
      raise Limite_Implantation; 
    end if;
    Dans.Longueur := Dans.Longueur + 1;
    Dans.Ieme (Dans.Longueur) := Val;
  end Prolonger;

  procedure Supprimer (Dans : in out Liste; 
                        En : in Positive) is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    Dans.Longueur := Dans.Longueur - 1;
    Dans.Ieme (En..Dans.Longueur) :=
                        Dans.Ieme (En + 1..Dans.Longueur + 1);
  end Supprimer;

  procedure Tronquer (Dans : in out Liste; 
                       En : in Natural) is
  begin
    if En > Dans.Longueur then 
      raise Erreur_Specification; 
    end if;
    Dans.Longueur := En;
  end Tronquer;

  procedure Copier (Dans : in out Liste; 
                     La_Liste : in Liste) is
  begin
    if Dans.Taille_Max < La_Liste.Longueur then
      raise Erreur_Specification;
    end if;
    Dans.Longueur := La_Liste.Longueur;
    Dans.Ieme (1..La_Liste.Longueur) := 
                         La_Liste.Ieme (1..La_Liste.Longueur);
  end Copier;

  function "=" (Left, Right : in Liste) return Boolean is
  begin
    return Left.Longueur = Right.Longueur
       and then Left.Ieme (1..Left.Longueur) = 
                               Right.Ieme (1..Right.Longueur);
  end "="; 

  function R_L is
    new Recherche_Contigue (Element => Element,
                            Cle => Cle, Table => Liste);

  function Recherche (Dans : in Liste; 
                       C : in Cle) return Positive is
  begin
    return R_L (Dans, C);
  end Recherche;

end Listes_Contigues_Ordonnees;
