--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--              CORPS DU PAQUETAGE DES TABLES               --
--                                                          --
-- Note: La finalisation effectue la desallocation despace --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Ada.Unchecked_Deallocation;
with Erreurs_Types_Abstraits; use Erreurs_Types_Abstraits;
package body Tables is
  type Place is         -- reprsentation effective des places
    record
      Numero : Integer;
      Contenu : Element;
      Suivant : Pt_Place;
    end record;
 
  procedure Liberer is 
            new Ada.Unchecked_Deallocation (Place, Pt_Place);
 
  function Ieme (De : in Table; 
                  En : in Integer) return Element is
    P : Pt_Place := De.Tete;
  begin            -- recherche de lՎlment qui doit tre dfini
    while P /= null and then P.Numero /= En loop
      P := P.Suivant;
    end loop;
    if P = null then raise Erreur_Specification; end if;
    return P.Contenu;
  end Ieme;

  procedure Changer_Ieme (De : in out Table; 
                           En : in Integer; 
                           Avec : in Element) is
    P : Pt_Place := De.Tete;
  begin
    if En < De.Bi or En > De.Bs then 
      raise Erreur_Specification;         -- test des bornes
    end if;
    if P = null then             -- cest le premier  crer
      De.Tete := new Place'(Numero => En, 
                            Contenu => Avec, 
                            Suivant => null);
    elsif P.Numero = En then    -- modification du premier
      P.Contenu := Avec;
    else                            -- rechercher le numro
      while P.Suivant /= null 
      and then P.Suivant.Numero /= En loop
        P := P.Suivant;
      end loop;
      if P.Suivant = null then            -- non encore dfini
        P.Suivant := new Place'(Numero => En, 
                                Contenu => Avec, 
                                Suivant => null);
      else                        -- modification de lexistant
        P.Suivant.Contenu := Avec;
      end if;
    end if;
  end Changer_Ieme;

  procedure Finalize (T : in out Table) is
    P : Pt_Place;
  begin
    while T.Tete /= null loop
      P := T.Tete;
      T.Tete := T.Tete.Suivant;
      Liberer (P);
    end loop;
  end Finalize;

end Tables;
